#!/bin/bash
# shellcheck disable=SC1091 # Cannot resolve paths in variables

PROJECT_HOME="$(realpath -s --relative-to . "$(dirname "$0")/..")" || exit
. "$PROJECT_HOME/scripts/util.sh" || exit

get_project_name
get_carch
get_product
get_project_type

IMAGE="${CARCH}/${PROJECT}"
IMAGE_NAME="localhost/${IMAGE}:latest"
IMAGE_SHORT_TAG="localhost/${PROJECT}:latest"
IMAGE_SRC="$PROJECT_HOME/container"
IMAGE_TAR="$PROJECT_HOME/swu/${PROJECT}.tar"

mode="$1"

build_image() {
	local mode="$1"
	local packages context="$IMAGE_SRC"
	podman rmi "$IMAGE_NAME" 2>/dev/null

	[ -e "$IMAGE_SRC/packages.txt" ] || error $"Could not read $IMAGE_SRC/packages.txt"
	if [ "$mode" == "build" ]; then
		while read line
		do
			if [[ "$line" =~ ^# ]]; then
				continue
			elif [ "$line" = "[build]" ]; then
				continue
			fi
			packages="$packages$line "
		done < "$IMAGE_SRC/packages.txt"
		IMAGE_NAME="localhost/${IMAGE}_build:latest"
	else
		while read line
		do
			if [[ "$line" =~ ^# ]]; then
				continue
			elif [ "$line" = "[build]" ]; then
				break
			fi
			packages="$packages$line "
		done < "$IMAGE_SRC/packages.txt"
	fi

	if [ "$PROJECT_TYPE" = "python" ]; then
		cp -f "$PROJECT_HOME/app/src/requirements.txt" "$IMAGE_SRC/resources"
	fi

	# Ignore for releases: build forbids symlinks outside of build
	# directory, so use external context for extension development.
	if [ -L "$IMAGE_SRC/resources" ]; then
		context="$PROJECT_HOME/../container_common"
	fi

	# podman 3.x as in debian bullseye does not support "copy if exist"
	# syntax with a glob, so if product resource directory does not exist
	# create an empty directory to avoid error
	local product_resources="$context/resources_${PRODUCT//"/"/_}"
	if ! [ -e "$product_resources" ]; then
		mkdir "$product_resources" ||
			error "Could not create $product_resources"
	fi

	podman build -t "$IMAGE_NAME" \
		--build-arg ARCH="$CARCH" \
		--build-arg PRODUCT="${PRODUCT//"/"/_}" \
		--build-arg PACKAGES="$packages" \
		-f "$IMAGE_SRC/Dockerfile" \
		"$context" ||
		error $"Could not build image"
}

save_image() {
	# We use the same tag on Armadillo regardless of arch,
	# just always retag
	podman tag "$IMAGE_NAME" "$IMAGE_SHORT_TAG"
	# delete old project container images
	podman image prune -f

	rm -f "$IMAGE_TAR"
	podman save -o "$IMAGE_TAR" "$IMAGE_SHORT_TAG" ||
		error $"Could not save image"

	echo $"Stored container image to $IMAGE_TAR"
}

build_image "$mode"

if ! [ "$mode" == "build" ]; then
	save_image
fi
